﻿using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Net;
using System.Security.Cryptography;
using System.Text;

namespace VMPInterface
{
    /**
     * VMP .NET版<下单示例代码> 
     * 公司: eftPay
     * 说明： 
     * 本示例只是简单编写alipay web 下单的示例，
     * vmp 所有接口签名方式都一样(vmp eopg除外)，
     * 其他场景的下单和查询、退款接口可以根据文档在本接口的基础上进行相关参数的改变就可以了。
     * 日期: 2022-06-06
     **/

    internal class VMPServicesHelper
    {

        //PRO还是UAT
        public static bool isDebug { get; set; }

        //vmp userkey
        public static string userKey { get; set; }
        //vmp 密钥
        public static string secret { get; set; }
    
        // vmp uat url
        public static string vmpUATUrl = "http://vmp.eftpay.com.cn/VMP/Servlet";
        //vmp pro url
        public static string vmpProdUrl = "开投产账户后提供";
        //请求类型
        private const string sContentType = "application/json";

        public VMPServicesHelper(string uKey, string uSecret, bool needDebug = false)
        {
            isDebug = needDebug;
            userKey = uKey;
            secret = uSecret;
        }
        /// <summary>
        /// 支付宝WEB PC下单接口
        /// </summary>
        /// <param name="outTradeNo">商户自定义单号</param>
        /// <param name="amount">金额</param>
        /// <param name="subject">交易描述</param>
        /// <param name="returnUrl">同步回调url</param>
        /// <param name="notifyUrl">异步通知URL</param>
        /// <param name="currency">币种</param>
        /// <returns></returns>
        public string AlipayWebPreOrder(string outTradeNo, string amount, string subject, string returnUrl = null, string notifyUrl = null, string currency = "HKD")
        {
            string service = "service.alipay.web.PreOrder";
            return WebPreOrder(service,  outTradeNo, amount, subject, returnUrl, notifyUrl, currency);
        }
        /// <summary>
        /// 下单接口
        /// </summary>
        /// <param name="service"></param>
        /// <param name="outTradeNo"></param>
        /// <param name="amount"></param>
        /// <param name="subject"></param>
        /// <param name="returnUrl"></param>
        /// <param name="notifyUrl"></param>
        /// <param name="currency"></param>
        /// <returns></returns>
        public string WebPreOrder(string service, string outTradeNo, string amount, string subject, string returnUrl = null, string notifyUrl = null, string currency = "HKD")
        {
            Dictionary<string, string> paymentPackage = new Dictionary<string, string>();
            //以下参数参考文档中的说明，每个场景参数有差异
            paymentPackage["service"] = service;
            paymentPackage["user_confirm_key"] = userKey;
            paymentPackage["transaction_amount"] = amount;
            paymentPackage["out_trade_no"] = outTradeNo;
            paymentPackage["buyerType"] = "ios";
            paymentPackage["subject"] = subject;
            paymentPackage["time"] = DateTime.Now.ToString("yyyyMMddHHmmss");

            if (service.Contains("alipay") || service.Contains("wechat"))
            {
                paymentPackage["payType"] = service.Contains("alipay") ? "Alipay" : "WeChat";
                paymentPackage["wallet"] = (service.Contains("alipay") ? "ALIPAYCN" : "WECHATCN");
                paymentPackage["pay_scene"] = (service == "service.wechat.oauth2.Authorize" ? "WXWEB" : (service == "service.alipay.wap.PreOrder" ? "WAP" : "WEB"));
            }

            if (service.Contains("atome"))
            {
                paymentPackage["payType"] = "ATOME";
                paymentPackage["wallet"] = "ATOME";
                paymentPackage["pay_scene"] = "ATOMEWEB";
            }

            paymentPackage["body"] = subject;

            paymentPackage["fee_type"] = currency;
            if (!string.IsNullOrEmpty(returnUrl))
            {
                paymentPackage["return_url"] = returnUrl;
            }
            if (!string.IsNullOrEmpty(notifyUrl))
            {
                paymentPackage["notify_url"] = notifyUrl;
            }
            //进行签名
            paymentPackage["sign"] = GenVmpSign(paymentPackage, secret);

            string url = isDebug ? vmpUATUrl : vmpProdUrl;

            // url 后跟的具体查看文档中“请求地址” ，这里是web接口示例
            string resp = HttpPost2(url + "/JSAPIService.do", JsonConvert.SerializeObject(paymentPackage), "POST", null, "application/json");

            return resp;
        }

        /// <summary>
        /// vmp 加密方法
        /// </summary>
        /// <param name="paymentPackage"></param>
        /// <param name="secret"></param>
        /// <returns></returns>

        private string GenVmpSign(Dictionary<string, string> paymentPackage, string secret)
        {
            Dictionary<string, string> orgDic = paymentPackage;
            Dictionary<string, object> unsortDic = JsonConvert.DeserializeObject<Dictionary<string, object>>(JsonConvert.SerializeObject(orgDic));

            var vDic = unsortDic.OrderBy(x => x.Key, new ComparerString()).ToDictionary(x => x.Key, y => y.Value);

            StringBuilder str = new StringBuilder();

            foreach (KeyValuePair<string, object> kv in vDic)
            {

                if (kv.Value != null)
                {
                    string pkey = kv.Key;
                    string pvalue = kv.Value.ToString();

                    str.Append(pkey + "=" + pvalue + "&");
                }

            }
            string signStr = str.ToString().Substring(0, str.ToString().Length - 1);
            string targetSign = SHA256Encode(secret + signStr, "UTF-8").ToLower();
            return targetSign;
        }

        public string SHA256Encode(string data, string signCharset = "GBK")
        {
            byte[] bytes = Encoding.GetEncoding(signCharset).GetBytes(data);
            byte[] hash = SHA256Managed.Create().ComputeHash(bytes);

            StringBuilder builder = new StringBuilder();
            for (int i = 0; i < hash.Length; i++)
            {
                builder.Append(hash[i].ToString("X2"));
            }
            return builder.ToString();
        }

        /// <summary>
        /// http请求接口api
        /// </summary>
        /// <param name="url">接口url</param>
        /// <param name="postDataStr">传递预支付操作路径接口</param>
        /// <param name="requesttype">请求方式（get/post）</param>
        /// <returns></returns>
        public static string HttpPost2(string url, string postDataStr, string requesttype = "POST", Dictionary<string, string> headers = null, string contentType = null)
        {
            HttpWebRequest request = null;


            string data = postDataStr;
            string type = requesttype;

            byte[] data_byte = Encoding.GetEncoding("UTF-8").GetBytes(data);
            Stream responseStream;
            if (url.StartsWith("https", StringComparison.OrdinalIgnoreCase))
            {
                //ServicePointManager.ServerCertificateValidationCallback = new RemoteCertificateValidationCallback(CheckValidationResult);
                ServicePointManager.Expect100Continue = true;
                ServicePointManager.SecurityProtocol = SecurityProtocolType.Tls12 | SecurityProtocolType.Tls11 | SecurityProtocolType.Tls | SecurityProtocolType.Ssl3;
                ServicePointManager.ServerCertificateValidationCallback = (sender, certificate, chain, errors) => true;


                request = WebRequest.Create(url) as HttpWebRequest;

                request.KeepAlive = false;
                request.UserAgent = "Mozilla/5.0 (Windows NT 6.1; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/59.0.3904.97 Safari/537.36";

            }
            else
            {
                request = WebRequest.Create(url) as HttpWebRequest;
            }
            if (request == null)
            {
                throw new ApplicationException(string.Format("Invalid url string: {0}", url));
            }

            if (contentType == null)
            {
                contentType = sContentType;
            }


            request.ContentType = contentType;
            string method = type;

            //request.Headers.Add("Authorization:Bearer ");
            if (headers != null)
            {
                foreach (KeyValuePair<string, string> item in headers)
                {
                    request.Headers.Add(item.Key, item.Value);
                }
            }


            if (method == "POST")
            {
                request.Method = method;

                request.ContentLength = data_byte.Length;
                Stream requestStream = request.GetRequestStream();
                requestStream.Write(data_byte, 0, data_byte.Length);
                requestStream.Close();
                try
                {
                    responseStream = request.GetResponse().GetResponseStream();
                }
                catch (Exception exception)
                {
                    throw exception;
                }
            }
            else
            {
                try
                {

                    responseStream = request.GetResponse().GetResponseStream();
                }
                catch (Exception exception)
                {
                    throw exception;
                }
            }
            string str = string.Empty;
            using (StreamReader reader = new StreamReader(responseStream, Encoding.GetEncoding("UTF-8")))
            {
                str = reader.ReadToEnd();
            }
            responseStream.Close();
            return str;
        }

    }
}
